/*

    File: analyse.c

    Copyright (C) 1998-2002  Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */
#include <stdio.h>
#include <string.h>
#ifndef  DJGPP
#include <sys/types.h>
#endif
#include "types.h"
#include "common.h"
#include "fnctdsk.h" /* get_LBA_part */
#include "hdaccess.h"
#include "analyse.h"
#include "bfs.h"
#include "swap.h"
#include "bsd.h"
#include "fat.h"
#include "ntfs.h"
#include "ext2.h"
#include "netware.h"
#include "intrface.h"
#include "rfs.h"
#include "lvm.h"
#include "md.h"

int search_type_0(t_param_disk disk_car,t_diskext partition,const int debug, const int dump_ind,const int paranoid)
{
  unsigned char buffer[8*0x200];
  if(hd_read_LBA(disk_car,8, &buffer, partition->lba)!=0)
    return -1;
  if((LBA2sector(disk_car,partition->lba)==disk_car->CHS.sector) && (LBA2head(disk_car,partition->lba)==disk_car->CHS.head))
  {	/* NTFS recovery using backup sector */
    if(recover_NTFS(disk_car,(struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind,1)==0)
    {
      strcpy(partition->info,"NTFS found using backup sector!");
      return 1;
    }
    return 0;
  }
  if(LBA2sector(disk_car,partition->lba)==(1+6))
  {	/* FAT32 recovery using backup sector */
    if(recover_FAT32(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind,1)==0)
    {
      strcpy(partition->info,"FAT found using backup sector!");
      return 1;
    }
    return 0;
  }
  if(recover_Linux_SWAP(disk_car,(const union swap_header *)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_LVM(disk_car,(const pv_disk_t*)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_FAT12(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_FAT16(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_FAT32(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind,0)==0) return 1;
  if(recover_HPFS(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_OS2MB(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
  if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind,0)==0) return 1;
  if(recover_netware((const struct disk_netware *)&buffer,partition)!=0) return 1;
  /* Because this superblock is at the end of the partition, you should be
   * in slow mode to find it... */
  if(recover_MD(disk_car,(const struct mdp_superblock_s*)&buffer,partition,debug,dump_ind)==0) return 1;
  return 0;
}

int search_type_1(t_param_disk disk_car,t_diskext partition,const int debug, const int dump_ind,const int paranoid)
{
  unsigned char buffer[8*0x200];
//  ecrit_rapport("search_type_1 lba=%ld\n",partition->lba);
  if(hd_read_LBA(disk_car,8, &buffer, partition->lba)!=0)
    return -1;
  if(recover_BSD(disk_car,(const struct disklabel *)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
  if(recover_BeFS(disk_car,(const struct disk_super_block *)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
  return 0;
}

int search_type_2(t_param_disk disk_car,t_diskext partition,const int debug, const int dump_ind,const int paranoid)
{
  unsigned char buffer[8*0x200];
//  ecrit_rapport("search_type_2 lba=%ld\n",partition->lba);
  if(partition->lba+8<disk_car->lba)
  {
    if(hd_read_LBA(disk_car,2, &buffer[0x400], partition->lba+2)==0)
    {
      if(recover_EXT2(disk_car,(const struct ext2_super_block*)&buffer[0x400],partition,debug,dump_ind)==0) return 1;
    }
  }
  if(paranoid)
  {
    int s_log_block_size;
    dword ref_lba=partition->lba;
    // try backup superblock
    // It must be in paranoid mode because it can hide otherwise other partition type
    // Block size: 1024, 2048 or 4096 bytes (8192 bytes on Alpha systems)
    // Assumes that TestDisk is not running under Alpha and s_blocks_per_group=8 * block size
    for(s_log_block_size=0;s_log_block_size<=2;s_log_block_size++)
    {
      // sparse superblock feature: The groups chosen are 0, 1 and powers of 3, 5 and 7.
      // Checking group 3
      const dword hd_offset=ref_lba+(3*(EXT2_MIN_BLOCK<<s_log_block_size)*8*(EXT2_MIN_BLOCK<<s_log_block_size)*512);
      if(hd_offset+8<disk_car->lba)
      {
	if(hd_read_LBA(disk_car,2, &buffer[0x400], hd_offset+2)==0)
	{
	  partition->lba=hd_offset;	// Will be corrected by the s_blocks_per_group
	  if(recover_EXT2(disk_car,(const struct ext2_super_block*)&buffer[0x400],partition,debug,dump_ind)==0)
	    return 1;
	  partition->lba=ref_lba;
	}
      }
    }
  }
  return 0;
}

int search_type_128(t_param_disk disk_car,t_diskext partition,const int debug, const int dump_ind,const int paranoid)
{
  if(partition->lba+128<disk_car->lba)
  {
	unsigned char buffer[8*0x200];
//	ecrit_rapport("search_type_128 lba=%ld\n",partition->lba);
	/* Test ReiserFS */
	if(hd_read_LBA(disk_car,8, &buffer, partition->lba+128)!=0) /* 64k offset */
	  return -1;
	if(recover_rfs(disk_car,(const struct reiserfs_super_block*)&buffer,partition,debug,dump_ind)==0) return 1;
  }
  return 0;
}

int check_part(t_param_disk disk_car,const int debug,t_diskext partition)  
{
  int ret=0;
  switch(partition->part_type)
  {
    case P_12FAT:
    case P_12FATH:
    case P_16FAT:
    case P_16FATH:
    case P_16FATBD:
    case P_16FATBDH:
    case P_32FAT:
    case P_32FAT_LBA:
    case P_32FATH:
    case P_32FAT_LBAH:
      ret=check_FAT(disk_car,partition,debug);
      break;
    case P_LINUX:
      ret=check_EXT2(disk_car,partition,debug);
      if(ret!=0)
	ret=check_rfs(disk_car,partition,debug);
      if(ret!=0)
      { aff_buffer(BUFFER_ADD,"No EXT2 or Reiser marker\n"); }
      break;
    case P_NTFSH:
    case P_NTFS:
      ret=check_NTFS(disk_car,partition,debug,0);
      break;
    case P_LVM:
      ret=check_LVM(disk_car,partition,debug);
      break;
    case P_BEOS:
      ret=check_BeFS(disk_car,partition,debug);
      break;
    case P_BSD:
      ret=check_BSD(disk_car,partition,debug);
      break;
    case P_LINSWAP:
      ret=check_Linux_SWAP(disk_car,partition,debug);
      break;
    case P_RAID:
      ret=check_MD(disk_car,partition,debug);
      if(ret!=0)
      { aff_buffer(BUFFER_ADD,"Invalid RAID superblock\n"); }
      break;
  }
  if(ret!=0)
  {
    ecrit_rapport("check_part %02X\n", partition->part_type);
    aff_part_buffer(2,disk_car,partition);
  }
  return ret;
}

