/*

    File: common.h

    Copyright (C) 1998-2002  Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */
#define SECTOR_SIZE     0x200

#define MAX_CYLINDERS 65535
#define MAX_HEADS 255
#define MAX_SECTORS 63


#define NBR_PART_MAX    40
#define NBR_DISK_MAX	40
#define TAB_PART 0x1BE
/* PARTITION TYPE */
#define P_NO_OS         0x00
#define P_12FAT         0x01
#define P_16FAT         0x04
#define P_EXTENDED      0x05
#define P_16FATBD       0x06
#define P_NTFS          0x07
#define P_HPFS          0x07
#define P_OS2MB         0x0A
#define P_32FAT         0x0B
#define P_32FAT_LBA     0x0C
#define P_16FATBD_LBA   0x0E
#define P_EXTENDX       0x0F
#define P_12FATH        0x11
#define P_16FATH        0x14
#define P_16FATBDH      0x16
#define P_NTFSH         0x17
#define P_32FATH        0x1B
#define P_32FAT_LBAH    0x1C
#define P_NETWARE	0x65
#define P_OLDLINUX      0x81
#define P_LINSWAP       0x82
#define P_LINUX         0x83
#define P_LVM		0x8E
#define P_BSD           0xA5
#define P_BEOS          0xEB
#define P_RAID		0xFD
enum { UP_UNK, UP_FAT12, UP_FAT16, UP_FAT32, UP_EXTENDED, UP_NTFS, UP_HPFS, UP_OS2MB, UP_NETWARE, UP_EXT2, UP_EXT3, UP_LVM, UP_BSD, UP_BEOS, UP_RAID, UP_LINSWAP, UP_LINSWAP2, UP_RFS, UP_RFS2 };
enum { STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG, STATUS_EXT, STATUS_EXT_IN_EXT};
enum {BAD_SS, BAD_ES, BAD_SH, BAD_EH, BAD_EBS, BAD_RS, BAD_SC, BAD_EC,
  BAD_SCOUNT};
int doprintf(const char *_format, ...) __attribute__ ((format (printf, 1, 2)));
int wdoprintf(WINDOW *window, const char *_format, ...) __attribute__ ((format (printf, 2, 3)));
int ecrit_rapport(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
void ecrit_rapport_string(const char *string,const int max_length);

typedef struct cellule_disk_param *t_param_disk;

typedef struct cellule_disk *t_disk;
typedef unsigned char t_sector[0x200];
typedef unsigned char t_entree[0x10];
typedef struct cellule_diskext *t_diskext;
typedef struct cellule_CHS t_CHS;

struct cellule_CHS
{
  unsigned int cylinder;
  unsigned int head;
  unsigned int sector;
};

typedef struct cellule_list_part * t_list_part;
struct cellule_list_part
{
  t_diskext part;
  t_list_part prev;
  t_list_part next;
  int to_be_removed;
};

struct cellule_disk_param
{
  unsigned int disk;
  dword lba;
  t_CHS CHS;	/* logical CHS */
#ifdef DJGPP
  t_CHS CHSR;	/* CHS low level */
  int mode;
  unsigned int nbr_err;
  int halt_on_errors;
#else
  char *name;
#endif
#ifdef LINUX
  unsigned int offset;
#endif
};

struct cellule_disk
 {
  byte disk;
  unsigned int cylinder;
  unsigned int head;
  unsigned int sector;
 };


struct cellule_diskext
 {
  unsigned int disk;
  dword lba;
  dword part_size;
  dword boot_sector;
  dword blocksize;
  unsigned char part_type;
  int upart_type;
  unsigned int status;
  unsigned int order;
  unsigned int errcode;
  char name[80];
  char info[80];
 };

typedef struct cellule_param t_param;
struct cellule_param
{
  word size;
  word info;
  dword cyl;
  byte head;
  byte sect;
  dword nbrsect1;
  dword nbrsect2;
  word  bps;
};

typedef struct str_fat t_fat_test;
struct str_fat
{
  dword offset;
  int f_fat12;
  int f_fat16;
  int f_fat32;
};
typedef struct struct_my_data t_my_data;
struct struct_my_data
{
  t_param_disk disk_car;
  t_diskext partition;
};

void dup_t_disk(t_disk, const t_disk);
void dup_t_diskext(t_diskext, const t_diskext);
int hd_identify(t_param_disk,const int debug,const int allow_partial_last_cylinder);
int ask_YN(WINDOW *window,const char*msg);
int read_line(void);
char read_char(const char *);
void *MALLOC(size_t size);
void FREE(void *ptr);
char read_key(void);
char test_key(void);
unsigned int up2power(const unsigned int number);
int hd_parse(t_param_disk param_disk[NBR_DISK_MAX],const int debug,const int allow_partial_last_cylinder);
void my_sort(void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *));
void set_part_name(t_diskext partition,const char *src,const int max_size);
int rebuild_FAT_BS(t_param_disk disk_car,t_diskext partition, const int debug, const int dump_ind,const int interface);


