/*

    File: fnctdsk.c

    Copyright (C) 1998-2002  Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "types.h"
#include "common.h"
#include "fnctdsk.h"
#include "lang.h"
#include "testdisk.h"
#include "hdaccess.h"
#include "intrface.h"
#include "analyse.h"
extern char* nom_os[256];

int bootable(const t_entree E)
{
  return (int)E[0];
}

dword sect_rel(const t_entree E)
{ return *((const dword*)&(E[8]));
}

dword nbr_sect(const t_entree E)
{ return *((const dword*)&(E[12]));
}

dword CHS2LBA(const t_param_disk disk_car,const t_CHS*CHS)
{ return ((dword)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1;
}

dword C_H_S2LBA(const t_param_disk disk_car,const word C, const byte H, const byte S)
{ return ((dword)C*(disk_car->CHS.head+1)+H)*disk_car->CHS.sector+S-1;
}

unsigned int LBA2sector(const t_param_disk disk_car, const dword sect)
{ return (sect%disk_car->CHS.sector)+1; }

unsigned int LBA2head(const t_param_disk disk_car, const dword sect)
{ return (sect/disk_car->CHS.sector)%(disk_car->CHS.head+1); }

unsigned int LBA2cylinder(const t_param_disk disk_car, const dword sect)
{ return (sect/disk_car->CHS.sector)/(disk_car->CHS.head+1); }

void LBA2CHS(const t_param_disk disk_car,const dword lba, t_CHS*CHS)
{
//  CHS->cylinder=LBA2cylinder(disk_car,lba);
//  CHS->head=LBA2head(disk_car,lba);
//  CHS->sector=LBA2sector(disk_car,lba);
  dword pos=lba;
  CHS->sector=(pos%disk_car->CHS.sector)+1;
  pos/=disk_car->CHS.sector;
  CHS->head=pos%(disk_car->CHS.head+1);
  CHS->cylinder=pos/(disk_car->CHS.head+1);
}

dword get_sect_s(const t_param_disk disk_car,const t_entree E)
{ return C_H_S2LBA(disk_car,(word)s_cyl(E),s_head(E),s_sect(E));
}

dword get_sect_e(const t_param_disk disk_car,const t_entree E)
{ return C_H_S2LBA(disk_car,(word)e_cyl(E),e_head(E),e_sect(E));
}

void dup_t_CHS(t_CHS * CHS_dest, const t_CHS * CHS_source)
{
  CHS_dest->cylinder=CHS_source->cylinder;
  CHS_dest->head=CHS_source->head;
  CHS_dest->sector=CHS_source->sector;
}

int compare2uint(unsigned int a, unsigned int b)
{
  if(a<b)
    return -1;
  else
    if(a==b)
      return 0;
    else
      return 1;
}

int compare_CHS(const t_CHS *CHS1, const t_CHS *CHS2)
{
  int res;
  res=compare2uint(CHS1->cylinder,CHS2->cylinder);
  if(res!=0)
    return res;
  res=compare2uint(CHS1->head,CHS2->head);
  if(res!=0)
    return res;
  return compare2uint(CHS1->sector,CHS2->sector);
}

void dup_t_param_disk(t_param_disk dest, const t_param_disk src)
{
  dup_t_CHS(&dest->CHS,&src->CHS);
  dest->lba=src->lba;
  dest->disk=src->disk;
#ifdef DJGPP
  dest->mode=src->mode;
  dest->nbr_err=src->nbr_err;
  dup_t_CHS(&dest->CHSR,&src->CHSR);
#else
  dest->name=src->name;
#endif
}

void dup_t_diskext(t_diskext dest, const t_diskext src)
{
  dest->lba=src->lba;
  dest->boot_sector=src->boot_sector;
  dest->blocksize=src->blocksize;
  dest->disk=src->disk;
  dest->part_type=src->part_type;
  dest->upart_type=src->upart_type;
  dest->part_size=src->part_size;
  dest->status=src->status;
  dest->order=src->order;
  strncpy(dest->info,src->info,sizeof(dest->info));
  strncpy(dest->name,src->name,sizeof(dest->name));
}

void partition2entree(const t_param_disk disk_car, const dword pos, const t_diskext partition, t_entree *entree)
{
  t_CHS start,end;
  LBA2CHS(disk_car,partition->lba,&start);
  LBA2CHS(disk_car,partition->lba+partition->part_size-1,&end);
  if(partition->status==STATUS_PRIM_BOOT)
    (*entree)[0]=(unsigned char)0x80;
  else
    (*entree)[0]=(unsigned char)0;             /* Non bootable */
  (*entree)[4]=partition->part_type;
  *((dword*)&((*entree)[8]))=partition->lba-pos;
  if(start.cylinder>1023)
  { /* Partition Magic 5 uses CHS=(1023,0,1) if extended or last logical *
     * Linux fdisk and TestDisk use CHS=(1023,lastH,lastS)               */
    (*entree)[1]=(unsigned char)disk_car->CHS.head;
    (*entree)[2]=(unsigned char)(disk_car->CHS.sector|((1023>>8)<<6));
    (*entree)[3]=(unsigned char)1023;
  }
  else
  {
    (*entree)[1]=(unsigned char)start.head;
    (*entree)[2]=(unsigned char)(start.sector|((start.cylinder>>8)<<6));
    (*entree)[3]=(unsigned char)(start.cylinder);
  }
  if(end.cylinder>1023)
  {
    (*entree)[5]=(unsigned char)disk_car->CHS.head;
    (*entree)[6]=(unsigned char)(disk_car->CHS.sector|((1023>>8)<<6));
    (*entree)[7]=(unsigned char)1023;
  }
  else
  {
    (*entree)[5]=(unsigned char)end.head;
    (*entree)[6]=(unsigned char)(end.sector|((end.cylinder>>8)<<6));
    (*entree)[7]=(unsigned char)end.cylinder;
  }
  *((dword*)&((*entree)[12]))=partition->part_size;
}

int entree2partition(const t_param_disk disk_car, const dword pos,t_diskext partition, const t_entree entree, const unsigned int status,const unsigned int order,const int debug)
{
  t_CHS start,end;
  t_CHS start_calculated,end_calculated;
  partition_reset(partition);
  partition->disk=disk_car->disk;
  partition->part_type=os(entree);
  partition->lba=pos+sect_rel(entree);
  partition->order=order;
  partition->part_size=nbr_sect(entree);

  LBA2CHS(disk_car,partition->lba,&start_calculated);
  LBA2CHS(disk_car,partition->lba+partition->part_size-1,&end_calculated);

  start.cylinder=(unsigned int)s_cyl(entree);
  start.head=(unsigned int)s_head(entree);
  start.sector=(unsigned int)s_sect(entree);
  end.cylinder=(unsigned int)e_cyl(entree);
  end.head=(unsigned int)e_head(entree);
  end.sector=(unsigned int)e_sect(entree);

  switch(status)
  {
    case STATUS_PRIM:
      if(partition->part_type==(unsigned char)P_EXTENDX || partition->part_type==(unsigned char)P_EXTENDED)
      {
	partition->status=STATUS_EXT;
	partition->upart_type=UP_EXTENDED;
      }
      else
	if(bootable(entree)!=0)
	  partition->status=STATUS_PRIM_BOOT;
	else
	  partition->status=status;
      break;
    default:
      partition->status=status;
      break;
  }
  /* Check CHS */
  if((start.sector==0)||(start.sector>disk_car->CHS.sector))
  {
    partition->errcode=BAD_SS;
    return 1;
  }
  if((end.sector==0)||(end.sector>disk_car->CHS.sector))
  {
    partition->errcode=BAD_ES;
    return 1;
  }
  if(start.head>disk_car->CHS.head)
  {
    partition->errcode=BAD_SH;
    return 1;
  }
  if(start.cylinder>disk_car->CHS.cylinder)
  {
    partition->errcode=BAD_SC;
    return 1;
  }
  if(end.head>disk_car->CHS.head)
  {
    partition->errcode=BAD_EH;
    return 1;
  }
  if(end.cylinder>disk_car->CHS.cylinder)
  {
    partition->errcode=BAD_EC;
    return 1;
  }
  /* */

  if(((start_calculated.cylinder<=1023)&& (get_sect_s(disk_car,entree)!=partition->lba))
    || ((start_calculated.cylinder>1023)&&(start.cylinder!=1023)&&(start.cylinder!=(start_calculated.cylinder&1023))))
  {
      ecrit_rapport("BAD_RS LBA=%ld %ld\n",partition->lba,get_sect_s(disk_car,entree));
      partition->errcode=BAD_RS;
      return 1;
  }

  if(((end_calculated.cylinder<=1023)&& (get_sect_e(disk_car,entree)!=partition->lba+partition->part_size-1))
     || ((end_calculated.cylinder>1023)&&(end.cylinder!=1023)&&(end.cylinder!=(end_calculated.cylinder&1023))))
  {
    partition->errcode=BAD_SCOUNT;
    return 1;
  }
  /* Check partition and load partition name */
  check_part(disk_car,debug,partition);
  return 0;
}

const char* errmsg_entree2partition(const unsigned int errcode)
{
  switch(errcode)
  {
    case BAD_SS: return m_BAD_S_SECT;
    case BAD_ES: return m_BAD_E_SECT;
    case BAD_SH: return m_BAD_S_HEAD;
    case BAD_EH: return m_BAD_E_HEAD;
    case BAD_EBS: return m_END_BFR_START;
    case BAD_RS: return m_BAD_RS;
    case BAD_SC: return m_BAD_S_CYL;
    case BAD_EC: return m_BAD_E_CYL;
    case BAD_SCOUNT: return m_BAD_SCOUNT;
  }
  return "";
}

int read_MBR(t_param_disk disk_car,void *buffer)
{
  if(hd_read_LBA(disk_car,1, buffer, 0))
  {
    doprintf(m_PART_RD_ERR);
    return 1;
  }
  return 0;
}

int write_MBR(t_param_disk disk_car,void *buffer)
{
  if(hd_write_LBA(disk_car,1, buffer, 0))
  {
    doprintf(m_PART_WR_ERR);
    return 1;
  }
  return 0;
}

t_list_part insert_new_partition(t_list_part list_part, t_diskext part)
{
  return insert_new_partition_aux(list_part, element_new(part));
}

t_list_part insert_new_partition_aux(t_list_part list_part, t_list_part new_element)
{ /* new partition musn't be used after insert !*/
  t_list_part prev=NULL,next;
  for(next=list_part;;next=next->next)
  { // prev new next
    if((next==NULL)||(new_element->part->lba<next->part->lba)||((new_element->part->lba==next->part->lba)&&(new_element->part->part_size<=next->part->part_size)))
    {
      if((next!=NULL)&&(next->part->lba==new_element->part->lba)&&(next->part->part_size==new_element->part->part_size)&&(next->part->part_type==new_element->part->part_type)&&(next->part->upart_type==new_element->part->upart_type))
      {
	FREE(new_element->part);
	FREE(new_element);
	return list_part;
      }
      else
      { /* prev new_element next */
	new_element->next=next;
	new_element->prev=prev;
	if(next)
	  next->prev=new_element;
	if(prev)
	{
	  prev->next=new_element;
	  return list_part;
	}
	return new_element;
      }
    }
    prev=next;
  }
  ecrit_rapport("insert_new_partition_aux : BUG!\n");
  exit(EXIT_FAILURE);
}

int check_list_part(t_list_part list_part)
{
  t_list_part prev=NULL,parts;
  if((list_part!=NULL) && list_part->prev)
  {
    ecrit_rapport("\ncheck_list_part error: list_part->prev!=NULL\n");
    exit(EXIT_FAILURE);
  }
  for(parts=list_part;parts;parts=parts->next)
  {
//	ecrit_rapport("%p %p %p\n",parts->prev, parts, parts->next);
	if(prev!=parts->prev)
	{
	  ecrit_rapport("\ncheck_list_part error: prev!=parts->prev\n");
	  exit(EXIT_FAILURE);
	}
	prev=parts;
  }
  if((prev!=NULL) && (prev->next!=NULL))
  {
    ecrit_rapport("\ncheck_list_part error: prev->next!=NULL\n");
    exit(EXIT_FAILURE);
  }
  return 0;
}

t_list_part sort_list_part(t_list_part list_part)
{
  t_list_part new_list_part=NULL,element,next;
  for(element=list_part;element!=NULL;element=next)
  {
    next=element->next;
    new_list_part=insert_new_partition_aux(new_list_part,element);
  }
  return new_list_part;
}

void delete_list_part(t_list_part list_part)
{
  t_list_part element;
  /* Libere la memoire */
  element=list_part;
  while(element!=NULL)
  {
    t_list_part next=element->next;
    FREE(element->part);
    FREE(element);
    element=next;
  }
}

void  partition_reset(t_diskext partition)
{
  partition->part_type=P_NO_OS;
  partition->upart_type=UP_UNK;
  partition->boot_sector=0;
  partition->blocksize=0;
  partition->status=STATUS_DELETED;
  partition->order=0;
  partition->part_size=0;
  partition->info[0]='\0';
  partition->name[0]='\0';
}

t_diskext partition_new(const unsigned int disk)
{
  t_diskext partition=(t_diskext)MALLOC(sizeof(*partition));
  partition->disk=disk;
  partition_reset(partition);
  return partition;
}

t_list_part element_new(t_diskext part)
{
  t_list_part new_element=(t_list_part)MALLOC(sizeof(*new_element));
  new_element->part=part;
  new_element->prev=new_element->next=NULL;
  new_element->to_be_removed=0;
  return new_element;
}

