#include <stdio.h>
#include <sys/types.h>
#include "types.h"
#include "common.h"
#include "fat.h"
int is_fat32(const int part_type)
{
  switch(part_type)
  {
    case P_32FAT:
    case P_32FAT_LBA:
    case P_32FATH:
    case P_32FAT_LBAH:
      return 1;
  }
  return 0;
}
int dump_fat_info(const struct fat_boot_sector*fh1, const int part_type)
{
  switch(part_type)
  {
    case P_12FAT:
      printf("FAT : 12\n");
      break;
    case P_16FAT:
      printf("FAT : 16\n");
      break;
    case P_32FAT:
      printf("FAT : 32\n");
      break;
    default:
      printf("No FAT\n");
      return 0;
  }
  printf("cluster_size %u\n", (unsigned int)fh1->cluster_size);
  if(sectors(fh1)!=0)
    printf("sectors      %u\n", sectors(fh1));
  if(fh1->total_sect!=0)
    printf("total_sect   %u\n", fh1->total_sect);
  if(is_fat32(part_type)!=0)
  {
    printf("fat32_length %u\n", fh1->fat32_length);
    printf("root_cluster %u\n", fh1->root_cluster);
    printf("flags        %04X\n", fh1->flags);
    printf("version      %u.%u\n", (unsigned int)fh1->version[0], (unsigned int)fh1->version[1]);
    printf("root_cluster %u\n", fh1->root_cluster);
    printf("info_sector  %u\n", fh1->info_sector);
    printf("backup_boot  %u\n", fh1->backup_boot);
  } else {
    printf("fat_length   %u\n", fh1->fat_length);
    printf("dir_entries  %u\n", get_dir_entries(fh1));
  }
  return 0;
}

unsigned int sector_size(const struct fat_boot_sector *fat_header)
{ return (unsigned int)(fat_header->sector_size[1]<<8)+(unsigned int)fat_header->sector_size[0]; }

unsigned int get_dir_entries(const struct fat_boot_sector *fat_header)
{ return (unsigned int)(fat_header->dir_entries[1]<<8)+(unsigned int)fat_header->dir_entries[0]; }

unsigned int sectors(const struct fat_boot_sector *fat_header)
{ return (unsigned int)(fat_header->sectors[1]<<8)+(unsigned int)fat_header->sectors[0]; }



int main()
{
  FILE *file_in;
  unsigned char buffer[10*0x200];
  unsigned int taille;
  file_in=fopen("BOOT_BIN.txt","rb");
  if(!file_in)
    return 1;
  taille=fread(buffer,1,10*0x200,file_in);
  fclose(file_in);
  if(memcmp(buffer,"0000 ",5)==0)
  {
    int pos_file;
    int pos_buffer2=0;
    unsigned char buffer2[3*0x200];
    for(pos_file=5;pos_file<taille;)
    {
      char string[3];
      if(buffer[pos_file]==' ')
	pos_file++;
      string[0]=buffer[pos_file];
      string[1]=buffer[pos_file+1];
      string[2]=0;
      printf("%02X",strtol(string,NULL,16));
      buffer2[pos_buffer2++]=strtol(string,NULL,16);
      if(pos_buffer2%16==0)
      {
	pos_file+=18;
	if(buffer[pos_file]==0xA || buffer[pos_file]==0xD)
	  pos_file++;
	if(buffer[pos_file]==0xA || buffer[pos_file]==0xD)
	  pos_file++;
	pos_file+=9;
      }
      else
	pos_file+=2;
    }
//    buffer2_size=pos_buffer2-1;
    memcpy(buffer,buffer2,3*0x200);
  }
  dump_fat_info(buffer,P_16FAT);
  return 0;
}
