/*

    File: rfs_dir.c

    Copyright (C) 1998-2002  Christophe GRENIER <grenier@cgsecurity.org>
    Some code from Yury Umanets <torque@ukrpost.net>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */
//#define OLD_REISER
#ifdef HAVE_REISERFS
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "dal/dal.h"
#ifdef OLD_REISER
#include "dal/file_dal.h"
#else
#include "dal/file.h"
#endif
#include "reiserfs/reiserfs.h"
#include "reiserfs/libprogs_tools.h"
#include "types.h"
#include "common.h"
#include "intrface.h"
#include "hdaccess.h"
#include "rfs.h"
#include "dir.h"
static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
typedef struct ext_dir ext2_file_data;
struct ext_dir
{
  char	name[MAX_NAME_LEN(DEFAULT_BLOCK_SIZE)];
  time_t modtime;
  __u16  mode;
  __u16  uid;
  __u16  gid;
  __u8   attr;
  __u32  start;		/* starting cluster number */
  __u64	 size;			/* file size (in bytes) */
};

struct list_dir_struct {
	t_file_data *dir_list;
	t_file_data *current_file;
	reiserfs_fs_t *current_fs;
	int flags;
};

static void reiser_write_display(WINDOW *window, const void*private_data);
static void reiser_write_rapport(const void*private_data);
static int reiser_is_directory(const void*private_data);
static long int reiser_get_inode(const void*private_data);

unsigned dal_get_blocksize(dal_t *dal) {
  if (!dal)
    return 0;
  return dal->blocksize;
}

char *dal_error(dal_t *dal) {
#ifdef OLD_REISER
    return "";
#else
    return dal->error;
#endif
}

static int file_read(dal_t *dal, void *buff, blk_t block, blk_t count) {
  dword off, blocklen;
  t_my_data *my_data=(t_my_data*)dal->data;
//  ecrit_rapport("reiser file_read(dal=%p,buff=%p,block=%ld, count=%ld)\n",dal,buff,block,count);
  if (!dal || !buff)
    return 0;
  off = (dword)block * ((dword)dal->blocksize/512);
  blocklen = (dword)count * ((dword)dal->blocksize/512);
//  ecrit_rapport("blocklen=%ld\n",blocklen);
  if(hd_read_LBA(my_data->disk_car,blocklen,buff,my_data->partition->lba+off))
    return 0;
  return 1;
}

static int file_write(dal_t *dal, void *buff, blk_t block, blk_t count)
{
  dword off, blocklen;
//  ecrit_rapport("reiser file_write\n");
  if (!dal || !buff)
    return 0;
  off = (dword)block * ((dword)dal->blocksize/512);
  blocklen = (dword)count * ((dword)dal->blocksize/512);
//  if(hd_write_LBA(my_data->disk_car,blocklen,buff,my_data->partition->lba+off))
    return 0;
//  return 1;
}

static int file_sync(dal_t *dal)
{
  if (!dal) return 0;
//  ecrit_rapport("reiser file_sync\n");
  return 1;
}

static int file_flags(dal_t *dal)
{
//  ecrit_rapport("reiser file_flags\n");
  if (!dal) return 0;
  return dal->flags;
}

static int file_equals(dal_t *dal1, dal_t *dal2)
{
//  ecrit_rapport("reiser file_equals\n");
  if (!dal1 || !dal2)
    return 0;
  return !strcmp((char *)dal1->data, (char *)dal2->data);
}

static unsigned int file_stat(dal_t *dal)
{
//  ecrit_rapport("reiser file_stat\n");
  if (!dal)
    return 0;
//  return (unsigned int)st.st_dev;
  return 1;
}

#ifdef OLD_REISER
static blk_t   file_len(dal_t *dal) {
#else
static count_t file_len(dal_t *dal) {
#endif
  dword max_off = 0;
  t_my_data *my_data=(t_my_data*)dal->data;
//  ecrit_rapport("reiser file_len\n");
  if (!dal) return 0;
  max_off=my_data->partition->part_size*512;
  return max_off / dal->blocksize;
}

static struct dal_ops ops = {
	file_read, 
	file_write, 
	file_sync, 
	file_flags, 
	file_equals, 
	file_stat, 
	file_len
};

dal_t *file_open(const char *dev, unsigned blocksize, int flags)
{
//  ecrit_rapport("reiser file_open\n");
  if (!dev) return NULL;
#ifdef OLD_REISER
  return dal_open(&ops, dev,blocksize, flags, (void *)dev);
#else
  return dal_open(&ops, blocksize, flags, (const void *)dev);
#endif
}

void file_close(dal_t *dal) {
//  ecrit_rapport("reiser file_close\n");
	if (!dal) return;
	dal_close(dal);
}

static int power_of_two(unsigned long value) {
	return (value & -value) == value;
}

#ifdef OLD_REISER
dal_t *dal_open(struct dal_ops *myops, const void *dev, size_t blocksize, int flags, void *data)
#else
dal_t *dal_open(struct dal_ops *myops, unsigned blocksize, int flags, void *data) 
#endif
{
	dal_t *dal;
//  ecrit_rapport("reiser dal_open\n");
	
	if (!myops) return NULL;
	
	if (!power_of_two(blocksize)) {
		fprintf(stderr, "Block size isn't power of two.\n");
		return NULL;
	}	
	
	if (!(dal = (dal_t *)MALLOC(sizeof(*dal))))
		return NULL;

	memset(dal, 0, sizeof(*dal));
	
	dal->ops = myops;
#ifdef OLD_REISER
#else
	dal->entity= NULL;
	strcpy(dal->name, "/dev/reiserfs");
	dal->error[0]=0;
#endif
	dal->data = data;
	dal->flags = flags;
	dal->blocksize = blocksize;
	
	return dal;
}

void dal_close(dal_t *dal) {
//  ecrit_rapport("reiser dal_close\n");
	
	if (!dal) return;
	
	dal->ops = NULL;
	dal->data = NULL;
	FREE(dal);
}

int dal_set_blocksize(dal_t *dal, unsigned blocksize) {

	if (!dal) return 0;
	
	if (!power_of_two(blocksize))
		return 0;
	
	dal->blocksize = blocksize;
	
	return 1;
}

size_t dal_blocksize(dal_t *dal) {

	if (!dal) return 0;

	return dal->blocksize;
}

int dal_read(dal_t *dal, void *buff, blk_t block, blk_t count) {

//  ecrit_rapport("reiser dal_read\n");
	if (!dal) return 0;

	if (dal->ops->read)
		return dal->ops->read(dal, buff, block, count);
	
	return 0;
}

int dal_write(dal_t *dal, void *buff, blk_t block, blk_t count) {

//  ecrit_rapport("reiser dal_write\n");
	if (!dal) return 0;
	
	if (dal->ops->write)
		return dal->ops->write(dal, buff, block, count);
		
	return 0;
}
	
int dal_sync(dal_t *dal) {

	if (!dal) return 0;

	if (dal->ops->sync)
		return dal->ops->sync(dal);
	
	return 0;	
}

int dal_flags(dal_t *dal) {

	if (!dal) return 0;

	if (dal->ops->flags)
		return dal->ops->flags(dal);
	
	return 0;
}

int dal_equals(dal_t *dal1, dal_t *dal2) {
//  ecrit_rapport("reiserfs dal_equals\n");
	
	if (!dal1 || !dal2) return 0;

	if (dal1->ops->equals)
		return dal1->ops->equals(dal1, dal2);
	
	return 0;	
}

#ifdef OLD_REISER
int dal_stat(dal_t *dal, struct stat *mystat)
{
//  ecrit_rapport("reiserfs dal_stat\n");
	if (!dal) return 0;
	if (dal->ops->stat)
		return dal->ops->stat(dal,mystat);
	return 0;
}
#else
unsigned int dal_stat(dal_t *dal) {
//  ecrit_rapport("reiserfs dal_stat\n");
	if (!dal) return 0;
	if (dal->ops->stat)
		return dal->ops->stat(dal);
	return 0;
}
#endif

blk_t dal_len(dal_t *dal) {
//  ecrit_rapport("reiserfs dal_len\n");
	
	if (!dal) 
		return 0;

	if (dal->ops->len)
		return dal->ops->len(dal);

	return 0;
}

t_file_data *reiser_dir(t_param_disk disk_car, const t_diskext partition, t_dir_data *dir_data, const unsigned long int cluster);
t_file_data *reiser_dir(t_param_disk disk_car, const t_diskext partition, t_dir_data *dir_data, const unsigned long int cluster)
{
  struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
  reiserfs_dir_t *dir;
  reiserfs_dir_entry_t entry;
  ls->dir_list=NULL;
  ls->current_file=NULL;
  if (!(dir = reiserfs_dir_open(ls->current_fs, "/"))) {
    aff_buffer(BUFFER_ADD,"Couldn't open dir\n");
    ecrit_rapport("Couldn't open dir\n");
    return NULL;
  }
//  ecrit_rapport("reiserfs_dir_open ok\n");
/*
  if (!reiserfs_dir_seek(dir, (uint32_t)offset)) {
    ecrit_rapport("Couldn't seek\n");
    aff_buffer(BUFFER_ADD,"Couldn't seek\n");
    goto error_free_dir;
  }
  */

  while (reiserfs_dir_read(dir, &entry))
  {
    t_file_data *new_file=MALLOC(sizeof(*new_file));
    ext2_file_data *new_ext_element=MALLOC(sizeof(*new_ext_element));
//    ecrit_rapport("reiserfs_dir_read ok\n");
    new_file->prev=ls->current_file;
    new_file->next=NULL;
    new_file->file_data=new_ext_element;
    /*
    new_ext_element->modtime=entry.de.i_mtime;
    new_ext_element->mode=inode.i_mode;
    new_ext_element->uid=inode.i_uid;
    new_ext_element->gid=inode.i_gid;
    new_ext_element->start=ino;
    new_ext_element->size=LINUX_S_ISDIR(inode.i_mode)?inode.i_size:
      inode.i_size| ((__u64)inode.i_size_high << 32);
      */
    memcpy(new_ext_element->name,entry.de_name,MAX_NAME_LEN(DEFAULT_BLOCK_SIZE));
    if(ls->current_file)
      ls->current_file->next=new_file;
    else
      ls->dir_list=new_file;
    ls->current_file=new_file;
    reiser_write_rapport(new_ext_element);
  }
// ecrit_rapport("reiser_dir done\n");
error_free_dir:
  reiserfs_dir_close(dir);
  return ls->dir_list;
}

static int reiser_is_directory(const void*private_data)
{
  const ext2_file_data *file=(const ext2_file_data*)private_data;
  return S_ISDIR(file->mode);
  return 0;
}

static long int reiser_get_inode(const void*private_data)
{
  const ext2_file_data *file=(const ext2_file_data*)private_data;
  return file->start;
}

static void reiser_write_rapport(const void*private_data)
{
  const ext2_file_data *file=(const ext2_file_data*)private_data;
  struct tm		*tm_p;
  char		datestr[80];
  if(file->modtime)
  {
    tm_p = localtime(&file->modtime);
    sprintf(datestr, "%2d-%s-%4d %02d:%02d",
	tm_p->tm_mday, monstr[tm_p->tm_mon],
	1900 + tm_p->tm_year, tm_p->tm_hour,
	tm_p->tm_min);
  } else {
    strcpy(datestr, "                 ");
  }
  ecrit_rapport("%6o  %5d  %5d   ", 
      file->mode, file->uid, file->gid);
  ecrit_rapport("%7lld", file->size);
  ecrit_rapport(" %s %s\n", datestr, file->name);
}

static void reiser_write_display(WINDOW *window, const void*private_data)
{
  const ext2_file_data *file=(const ext2_file_data*)private_data;
  struct tm		*tm_p;
  char		datestr[80];
  if(file->modtime)
  {
    tm_p = localtime(&file->modtime);
    sprintf(datestr, "%2d-%s-%4d %02d:%02d",
	tm_p->tm_mday, monstr[tm_p->tm_mon],
	1900 + tm_p->tm_year, tm_p->tm_hour,
	tm_p->tm_min);
  } else {
    strcpy(datestr, "                 ");
  }
  /*
  wdoprintf(window, "%6o  %5d  %5d   ", 
      file->mode, file->uid, file->gid);
  wdoprintf(window, "%7lld", file->size);
  wdoprintf(window, " %s ", datestr);
  */
  wdoprintf(window, "%s\n", file->name);
}

int dir_partition_reiser(WINDOW *window,t_param_disk disk_car,t_diskext partition, const int debug)
{
  dal_t *dal;
  reiserfs_fs_t *fs;
  reiserfs_path_node_t *leaf;
  t_my_data my_data;
  int32_t offset = 0;
  int error = 0;

  aff_buffer(BUFFER_RESET,"Q");
  wmove(window,4,0);
  aff_part(window,1,disk_car,partition);
  aff_part_rapport(1,disk_car,partition);
  ecrit_rapport("\n");
  my_data.partition=partition;
  my_data.disk_car=disk_car;

  if (!(dal = file_open(&my_data, DEFAULT_BLOCK_SIZE, O_RDONLY))) {
    ecrit_rapport("Couldn't open device\n");
    aff_buffer(BUFFER_ADD,"Couldn't open device\n");
    goto error;
  }
//  ecrit_rapport("file_open ok\n");

  if (!(fs = reiserfs_fs_open_fast(dal, dal))) {
    ecrit_rapport("Couldn't open reiser filesystem %s\n",dal_error(dal));
    aff_buffer(BUFFER_ADD,"Couldn't open reiser filesystem\n");
    goto error_free_dal;
  }
//  ecrit_rapport("reiserfs_fs_open_fast ok\n");
  {
    struct list_dir_struct ls;
    t_dir_data dir_data;
    ls.dir_list=NULL;
    ls.current_file=NULL;
    ls.current_fs=fs;
    ls.flags = 0; //DIRENT_FLAG_INCLUDE_EMPTY;
    dir_data.window=window;
    dir_data.write_rapport=reiser_write_rapport;
    dir_data.write_display=reiser_write_display;
    dir_data.is_directory=reiser_is_directory;
    dir_data.get_cluster_inode=reiser_get_inode;
    dir_data.debug=debug;
    dir_data.private_dir_data=&ls;
    dir_data.get_dir=reiser_dir;
    dir_partition(disk_car,partition,&dir_data,2);
  }
  reiserfs_fs_close(fs);
  file_close(dal);
  return 0;
error_free_dal:
  file_close(dal);
error:
  aff_buffer(BUFFER_DISPLAY,"Q",window);
  return 0xff;    
}

#else
#include "types.h"
#include "common.h"
#include "intrface.h"
#include "hdaccess.h"

int dir_partition_reiser(WINDOW *window,t_param_disk disk_car,t_diskext partition)
{
  aff_buffer(BUFFER_RESET,"Q");
  move(5,0);
  aff_part(window,0,disk_car,partition);
  aff_part_rapport(0,disk_car,partition);
  aff_buffer(BUFFER_ADD,"Recompile with E2FS library");
  aff_buffer(BUFFER_DISPLAY,"Q",window);
  return 0;
}
#endif
